@@ -462,6 +462,10 @@ static const struct ipset_attr_policy create_attrs[] = {
 		.type = MNL_TYPE_U32,
 		.opt = IPSET_OPT_MEMSIZE,
 	},
+	[IPSET_ATTR_BITMASK] = {
+		.type = MNL_TYPE_NESTED,
+		.opt = IPSET_OPT_BITMASK,
+	},
 };
 
 static const struct ipset_attr_policy adt_attrs[] = {
@@ -1721,6 +1725,10 @@ rawdata2attr(struct ipset_session *session, struct nlmsghdr *nlh,
 	if (attr->type == MNL_TYPE_NESTED) {
 		/* IP addresses */
 		struct nlattr *nested;
+
+		if (type == IPSET_ATTR_BITMASK)
+			family = ipset_data_family(session->data);
+
 		int atype = family == NFPROTO_IPV4 ? IPSET_ATTR_IPADDR_IPV4
 					      : IPSET_ATTR_IPADDR_IPV6;
 
@@ -1982,7 +1990,7 @@ build_msg(struct ipset_session *session, bool aggregate)
 		break;
 	case IPSET_CMD_ADD:
 	case IPSET_CMD_DEL: {
-		const struct ipset_type *type;
+		DD(const struct ipset_type *type);
 
 		if (!aggregate) {
 			/* Setname, type not checked/added yet */
@@ -2007,7 +2015,7 @@ build_msg(struct ipset_session *session, bool aggregate)
 				open_nested(session, nlh, IPSET_ATTR_ADT);
 			}
 		}
-		type = ipset_data_get(data, IPSET_OPT_TYPE);
+		DD(type = ipset_data_get(data, IPSET_OPT_TYPE));
 		D("family: %u, type family %u",
 		  ipset_data_family(data), type->family);
 		if (open_nested(session, nlh, IPSET_ATTR_DATA)) {
@@ -2027,7 +2035,7 @@ build_msg(struct ipset_session *session, bool aggregate)
 		break;
 	}
 	case IPSET_CMD_TEST: {
-		const struct ipset_type *type;
+		DD(const struct ipset_type *type);
 		/* Return codes are not aggregated, so tests cannot be either */
 
 		/* Setname, type not checked/added yet */
@@ -2040,7 +2048,7 @@ build_msg(struct ipset_session *session, bool aggregate)
 			return ipset_err(session,
 				"Invalid test command: missing settype");
 
-		type = ipset_data_get(data, IPSET_OPT_TYPE);
+		DD(type = ipset_data_get(data, IPSET_OPT_TYPE));
 		D("family: %u, type family %u",
 		  ipset_data_family(data), type->family);
 		ADDATTR_SETNAME(session, nlh, data);
